Dragon60_import = function (dragonfile = "", makeNA = TRUE, outfile = "")     {

# Import of a Dragon 6.0 descriptor file set (*.txt) into matrix x and RData-file

#  ARGUMENTS
#  =========

#  dragonfile        Dragon descriptor file set (*.txt)
#                    (basic filename, without extension, see below)

#                    Export of descriptors by Dragon is done into 3 files:

#                     <basic filename>.TXT      descriptor values (only data)
#                     <basic filename>_descriptors.TXT    descriptor names
#                     <basic filename>_moleculaes.TXT     molecule names 
#                                                             (from Molfiles)

#                      Missing values = -9999 (user-setting in Dragon)

#                      See other Dragon settings in documentation

#  makeNA            TRUE: missing values (encoded by Dragon with -9999)
#                          are transformed to NA

#  outfile           RData output file with the generated descriptor matrix
#                      including row names = molecule names and
#                                col names = descriptor names
#                      if NOT ""

#                    load(<outfile>.RData) gives matrix object x
                     
#  VALUES
#  ======

#  x                 matrix with descriptors (also in RData file)

 
version = "Dragon60_import 130429"        # K. Varmuza, TU Vienna
                                          
comp_time = proc.time()      

# Parameter fr Dragon file

NAold = -9999   # missing values
 
cat("\nStart: ",version)
cat("\nDragon-file set basic name: ",dragonfile)
cat("\nMissing values (",NAold,") replaced by NA:",makeNA)
cat("\nOutput file with descriptor matrix:",outfile)
 
# Read file with descriptor names

file_descr_names = paste(dragonfile,"_descriptors.TXT",sep="")
descr_names = scan(file=file_descr_names,what="character",sep="\r",quiet=TRUE)
m = length(descr_names)
cat("\nDescriptor names file:",file_descr_names,"read with",m,"descriptor names:")
cat("\n  Examples [1...3]: ",descr_names[1:3])
  
# Read file with molecule names

file_mol_names = paste(dragonfile,"_molecules.TXT",sep="")
mol_names = scan(file=file_mol_names,what="character",sep="\n",quiet=TRUE)
n = length(mol_names)
cat("\nMolecular names file:",file_mol_names,"read with",n,"molecule names:")
cat("\n  Examples [1...2]: ",mol_names[1:2])

# Read file with descriptor values

file_descr = paste(dragonfile,".TXT",sep="")
x = as.matrix(read.table(file=file_descr,nrow=n))   # corr. 130429 ====
nx = nrow(x)
mx = ncol(x)
rownames(x) = mol_names
colnames(x) = descr_names

cat("\nDescriptor file:",file_descr,"read with X(",n,"x",m,")")

# Replace NAold (missing values) by NA

if (makeNA) {
 for (i in 1:n) {
  for (j in 1:m) {if (x[i,j]==NAold) {x[i,j]=NA}}
 }
 if_x_NA = is.na(x)		# logical matrix for NA
 nNA = sum(if_x_NA)
 cat("\n",NAold,"for missing values replaced by NA")
 cat("\n",nNA," NAs in descriptor matrix")
}

# RData output file

if (outfile != "") {
    save(x,file=outfile)
    cat("\nDescriptor file made:",outfile) }
                                           

comp_time = proc.time()-comp_time
cat("\nEnd of",version, "  ",comp_time[1],"s \n")

return(x)

}  # end function
 