go_rdcv = function (            
            title="no_title",
            X, y,
            method="simpls",
            amax=15,
            repetitions=20,
            seg_test=4,
            seg_calib=7,
            parsimony_factor=1,
            PDFfile="rdCV_plots.PDF",
            messages=1) {


# Go to: easy rdCV (repeated double Cross Validation)
# ==================================================

# This is an easy call of function mvr_dcv (in package 'chemometrics 0.8').
# mvr_dcv has been slightly modified and is used as "mvr_dcv_VK.R".

# Method: Evaluation of PLS regression models y = f(X) by rdCV
#         y: only one response  

# Author:         K. Varmuza (TU Vienna, Austria)
#                 (sorry, an unexperienced R-programmer)
# Contributors:   P. Filzmoser, B. Liebmann (TU Vienna, Austria)
	
# Ref.:   Filzmoser P., Liebmann B., Varmuza K.:
#         J. Chemometrics 23, 160-171 (2009)
#         Repeated double cross validation

#         Varmuza K., Filzmoser P.:
#         Introduction to multivariate statistical analysis in chemometrics;
#         CRC Press, Boca Raton, FL, USA (2009)

# version = "go_rdcv 140114a VK"   # corrected for R 3.0.2
  version = "go_rdcv 140214a VK"   # error in boxplots with SEP values corrected


# ARGUMENTS (Input)
# =================

# title         text string with max. 20 characters (appears in outputs)
#               (def. = "no_title")

# X             X-Matrix (rows = objects (samples), columns = variables)
# y             y-vector (a single property to be modeled by X)

# method        multiple regression method as implemented in function "mvr"
#                 in package "pls" (by B.H. Mevik, R. Wehrens: 
#                 J. Statistical Software 18 [2] 2007);
#                 "kernelpls", "simpls", "oscorepls", "svdpc";
#               (def. = "simpls")

# amax          maximum desired no. of PLS components (def. = 15)

# repetitions   no. of repetitions in rdCV (def. = 20)

# seg_test      no. of segments in outer CV loop (random split into calibration
#                 sets and test sets), in mvr_dcv: variable "segments0"; 
#               (def. = 4)

# seg_calib     no. of segments in inner CV loop (CV with random split for  
#                 optimizing the no. of PLS components), 
#                 in mvr_dcv: variable "segments"; 
#               (def. = 7)

# parsimony_factor   for "standard error method" used for optimizing
#                    the no. of PLS components 
#                    (according to Hastie T., Tibshirani R.J., Friedman J.:
#                    The elements of statistical learning; 
#                    Springer: New York, NY, USA, 2001);
#                    in mvr_dcv: variable "sdfact" 
#                    (def. = 1)

# PDFfile       name of output PDF file (2 pages A4) with diagnostic plots;
#                 no plot if "";
#               (def.: = "rdCV_plots.PDF") 

# messages      <=0: no output on screen, >0: some messages on screen;
#               (def. = 1) 


# VALUE (Output)
# ==============

# title       title (= input title, however, only first 20 characters)

# method      PLS method (= input method)
# rep         no. of repetitions (= input repetitions)
# seg_test    no. of segments in outer CV loop (split into calibration sets
#                 and test sets, = input seg_test)
# seg_calib   no. of segments in inner CV loop (CV for optimizing the no.
#                 of PLS components, = input seg_calib) 
# parsimony   parameter for estimation of optimum no. of PLS components
#             (= input parsimony_factor)

# n            no. of objects (samples) (= no. of rows in X or y)
# m            no. of variables         (= no. of columns in X)

# amax         no. of PLS components calculated (maybe smaller than input amax)

# a_final      final optimum no. of PLS components (a single number); 
#              in mvr_dcv: variable "afinal" 

# a_opt_all    all optimal no.s of PLS components obtained during rdCV;
#               matrix [(segTEST) x (repetitions)];
#              in mvr_dcv: matrix "optcomp"

# ypred_all    all test set predicted y-values obtained during rdCV 
#               3-dimensional array [(n) x (amax) x (repetitions)];
#              in mvr_dcv: array "pred[,1,,]" 

#               Note: ypred_all[1:n, a_final, 1:repetitions] 
#                refers to the final optimum no. of PLS components, a_final.
#               Note: The predicted y's are obtained from different models
#                during rdCV.

# yres_all      all residuals obtained during rdCV (corresponding to ypred_all);
#                3-dimensional array [(n) x (amax) x (repetitions)];
#               in mvr_dcv no explicit object

# yres_opt_quant    0.025 and 0.975 quantiles of all residuals at a_final;
#                   vector yres_opt_quant[1:2]

# SEP_all       SEP for each number of PLS components and each repetition;
#                matrix [(repetitions) x (amax)];
#               in mvr_dcv no explicit value; is calculated from "pred" 

# SEP_final     final SEP (a single number, calculated from 
#                all [(n) x (repetitions)] residuals, 
#                for a_final PLS components;
#               in mvr_dcv: variable "SEPfinal[afinal]"

# bias_all      bias values for all repetitions and all no. of components;
#                matrix [(repetitions) x (amax)] 	

# bias_opt_mean  mean of bias values at a_final 
#                 = bias from all residuals at a_final

# bias_opt_quant  0.025 and 0.975 quantiles of all bias values at a_final;
#                 vector bias_opt_quant[1:2]

# corr_all        Pearson corr. coeff. between y and predicted y;
#                  for each repetition and each no. of components;
#                  matrix [(repetitions) x (amax)] 	

# corr_opt_mean   mean Pearson corr. coeff. between y and predicted y;
#                  at a_final 

# corr_opt_quant  0.025 and 0.975 quantiles of all Pearson corr. coeff.
#                   between y and predicted y at a_final;
#                 vector corr_opt_quant[1:2]


# USED R-software
# ===============

#  Package "chemometrics" and packages called therein;

#  mvr_dcv_VK.R, text2PDF.R (both provided as separate R-source files)

################################################################################
  

comptime = proc.time()              # computing time init.

# Prepare input
# -------------

# No checks of input parameters currently implemented

title_rdcv = substr(title,1,20)     # take only first 20 characters of title
if (messages>0) {
   cat("\n=== Start of ",version,"   ", date())
   cat("\n Title used: ",title_rdcv, "\n")        }

library(chemometrics)
source("mvr_dcv_VK.R")         # adapted version of mvr_dcv() 

n = nrow(X)
m = ncol(X)
ny=length(y)
ymin = min(y)
ymax = max(y)
ysd = sd(y)
amax_pls = min(amax,n-1,m)      # actual no. of components computed

if (messages>0) {
   cat(" X: ",n," rows   ", m,"cols")
   cat("\n y: ",ny," values   ",round(ymin,4),"-",round(ymax,4),"   sd =",
                                round(ysd,3))
   cat("\n No. of PLS components:",amax,"desired")}

Xy = data.frame(X=X, y=y)       # prepare input data for mvr_dcv2_VK

# Call mvr_dcv_VK (performs rdCV)
#--------------------------------
set.seed(1234)
res = mvr_dcv_VK (y~X, data=Xy, 
                  method=method, ncomp=amax_pls,
                  repl=repetitions,
                  segments0=seg_test, segments=seg_calib,
                  sdfact=parsimony_factor,
                  trace=FALSE, plot.opt=FALSE)

# Prepare output of go_rdcv 
# -------------------------

amax_pls = length(res$SEPfinal)  # no. of PLS components from mvr_dcv2_VK,
                                 # for the case the value was modified
if (messages>0) {cat("  ",amax_pls,"computed")
                 cat("\n rdCV:  segments:",seg_test,seg_calib,
                     " repetitions:",repetitions," parsimony:",parsimony_factor) 
  }

a_final = res$afinal             # final optimum no. of PLS components

a_opt_all = res$optcomp    # all values for optimum no.s of PLS components;
                           # matrix [(segTest) x repetitions)]

ypred_all = res$pred[,1,,]       # 3D array for all predicted y's from rdCV 
yres_all = ypred_all - y         # and residuals.
                                 # Indices: [objects, components, repetitions]

SEP_all = matrix(0, nrow=amax_pls, ncol=repetitions)  
                                 # init matrix for SEP for all no. of components
                                 # and all repetitions (sd calculated over n 
                                 # objects), [components, repetitions]

for (a in 1:amax_pls) {
  for (r in 1:repetitions) { SEP_all[a,r] = sd(yres_all[,a,r]) } }
                         
SEP_final = res$SEPfinal[res$afinal]  # a single number, 
                                      # from [(n) x (repetitions)] residuals,
                                      # for a_final components

yres_opt_mean = mean(yres_all[,a_final,])  # mean of residuals at a_final

yres_opt_quant = quantile(yres_all[,a_final,],c(0.025, 0.975))
                    # 0.025 and 0.975 quantiles of all residuals at a_final;
                    # yres_opt_quant[1:2]

bias_all = colMeans(yres_all)  
bias_all = t(bias_all)    # bias values for all repetitions and all
                          # no. of components;
                          # matrix [repetitions, components] 	

bias_opt_mean = mean(bias_all[,a_final])  # mean of bias values at a_final
                                          # = bias from all residuals at a_final

bias_opt_quant = quantile(bias_all[,a_final], c(0.025, 0.975))  
                     # 0.025 and 0.975 quantiles of all bias values at a_final;
                     # bias_opt_quant[1:2]

corr_all = matrix(NA,nrow=repetitions,ncol=amax_pls)  # Pearson corr. coeff.
                              # between y and predicted y (for each repetition
                              # and each no. of components
  for (i3 in 1:repetitions)  {
       for (i2 in 1:amax_pls) {
           corr_all[i3,i2] = cor(y, ypred_all[,i2,i3]) } }

corr_opt_mean = mean(corr_all[,a_final])  # mean of corr.coeff. at a_final

corr_opt_quant = quantile(corr_all[,a_final], c(0.025, 0.975))  
                     # 0.025 and 0.975 quantiles of all corr.coeff. at a_final;
                     # corr_opt_quant[1:2]

if (messages>0) {
    cat("\n SEP_final = ",round(SEP_final,3)," at a_final =",a_final,
        "   R2 =",round(corr_opt_mean,4))
}

result = list(  title=title_rdcv, method=method,
                rep=repetitions,
                seg_test=seg_test, seg_calib=seg_calib,
                parsimony=parsimony_factor,
                n=n, m=m,
                amax=amax_pls,
                a_final=a_final,
                a_opt_all=a_opt_all,
                ypred_all=ypred_all,
                yres_all=yres_all,
                yres_opt_quant=yres_opt_quant,
                SEP_all=SEP_all,
                SEP_final=SEP_final,
                bias_all=bias_all,
                bias_opt_mean=bias_opt_mean,
                bias_opt_quant=bias_opt_quant,
                corr_all=corr_all,
                corr_opt_mean=corr_opt_mean,
                corr_opt_quant=corr_opt_quant) 
                

# PDF (2 A4) with selected results and diagnostic plots
# -----------------------------------------------------

if (PDFfile != "") {
#~~~~~~~~~~~~~~~~~~~~~~

 pdf(file=PDFfile,paper="special",pagecentre=TRUE,width=6.5,height=9.9)
 par(mfrow = c(3,2))        # 3 rows each with 2 plots  
 if (messages>0) {cat("\n PDFfile",PDFfile, "opened")}
 source("text2PDF.R")

# Text box 1 (left)
# -----------------

n_lines = 12           # no. of lines in following text output
t=rep("",n_lines)  
t[1] = paste("Title: ",title_rdcv)
t[2] = "* rdCV evaluation of PLS models *"
t[3] = " "
t[4] = paste("X:   n =",n,"  m =",m)
t[5] = paste("y:", round(ymin,3),"to",round(ymax,3),"  sd =",
                   round(ysd,3))
t[6] = " "
t[7] = paste("No. segments: outer CV =",seg_test,
                        "   inner CV =",seg_calib)
t[8] = paste("No. repetitions =",repetitions)
t[9] = paste("Parsimony factor =",parsimony_factor)
t[10] = paste("Method =",method)
t[11] = " "
t[12] = PDFfile
text2PDF(t,width=8,height=6.5,line_height=0.5,box=FALSE)

# Text box 2 (right)
# ------------------

n_lines = 12
t=rep("",n_lines) 
t[1] = date()
t[2] = paste("R-function: ",version)
t[3] = "  "
t[4] = paste("Final opt. no. PLS components =",a_final)
t[5] = paste("SEP final =",round(SEP_final,3))
t[6] = paste("RESIDUALS: mean =",round(yres_opt_mean,3))
t[7] = paste("  95% interval    =",round(yres_opt_quant[1],3),"to",
                                   round(yres_opt_quant[2],3))
t[8] = paste("  +-2*SEP_final =",round((-2*SEP_final),3),"to",
                                 round((2*SEP_final),3))
t[9] =  paste("BIAS: mean =",round(bias_opt_mean,3))
t[10] = paste("  95% interval =",round(bias_opt_quant[1],3), "to",
                                 round(bias_opt_quant[2],3))
t[11] = paste("CORR.COEFF.: mean =",round(corr_opt_mean,4))
t[12] = paste("  95% interval    =",round(corr_opt_quant[1],4),"to",
                                    round(corr_opt_quant[2],4))  
text2PDF(t,width=8,height=6.5,line_height=0.5,box=FALSE)


# Plot 1: Predicted y versus experimental y (at a_final)
# ------------------------------------------------------

#  ypred_all     3D array for all predicted y's from rdCV 
#                 [objects, components, repetitions]

ypred_mean = rep(NA,n)  # init mean vector (at a_final, mean of all repetitions
for (i in 1:n) { ypred_mean[i]=mean(ypred_all[i,a_final,]) }

plot(0, 0, main="y-predicted versus y-experimental",
        xlab="Experimental y", ylab="Predicted y",
        cex.lab=1.2, cex=0.7, pch=3, col="black", type="n",
        xlim = c(ymin,ymax),
        ylim = c(min(ypred_all[,a_final,]), max(ypred_all[,a_final,])) )
for (i in 1:n) {
  for (r in 1:repetitions) {
     points(y[i], ypred_all[i,a_final,r], cex=0.3, pch=3, col=gray(0.6)) }
  points(y[i], ypred_mean[i], cex=0.7, pch=3, col="black")
}

abline(c(0,1), lty = 1)

# Plot 2: Residuals versus object/sample number (at a_final) 
# ---------------------------------------------------------

#  yres_all    3D array for all residuals from rdCV
#               [objects, components, repetitions]

yres_mean = rep(NA,n)  # init mean vector (at a_final, mean of all repetitions
for (i in 1:n) { yres_mean[i]=mean(yres_all[i,a_final,]) }

plot(0, 0, main="Residuals of objects",
         xlab="Object number", ylab="Residuals",
         cex.lab=1.2, cex=0.7, pch=3, col="black", type="n",
         xlim = c(1,n),
         ylim = c(min(yres_all), max(yres_all)) )

for (i in 1:n) {
  for (r in 1:repetitions) {
     points(i, yres_all[i,a_final,r], cex = 0.3, pch = 3, col = gray(0.6)) }
  points(i, yres_mean[i], cex = 0.7, pch = 3, col = "black")
}

abline(h = 0, lty = 1)
abline(h = 2*SEP_final,lty=2)
abline(h = -2*SEP_final,lty=2)

# Plot 3: Density distribution of residuals 
# -----------------------------------------
# R-code adapted from function plotdensitymvr1.R
#  (P. Filzmoser, package "chemometrics")

    resdcvall <- res$pred[, , a_final, ] - y
    resmean <- apply(resdcvall,1,mean)
    densmean <- density(resmean)
    densall  <- density(resdcvall)

    k <- length(densmean$x) # number of density estimated points
    nrep=repetitions
    xmat <- matrix(NA,nrow=k,nrep)
    ymat <- matrix(NA,nrow=k,nrep)
    for (i in 1:nrep){
      dens <- density(resdcvall[,i])
      xmat[,i] <- dens$x 
      ymat[,i] <- dens$y   }

    xlimits <- range(cbind(xmat,densmean$x))
    ylimits <- range(cbind(ymat,densmean$y))

    plot(densmean$x,densmean$y,main="Distribution of residuals",
         xlab="Residuals (0.025 and 0.975 quantiles)",
         ylab="Density",
         cex.lab = 1.2,type = "n",xlim = xlimits,ylim = ylimits)
    for (i in 1:nrep) {
        lines(xmat[,i], ymat[,i], lwd = 0.3, col = gray(0.6))
    }
    lines(densmean$x,densmean$y,col=1,lty=1)
    abline(v = yres_opt_quant[1], lty = 2)  # 0.025 quantile of all
                                            #  residuals at a_final 
    abline(v = yres_opt_quant[2], lty = 2)  # 0.975 quantile

# Plot 4: Frequencies of optimum a-values (no. of PLS comp.)
# ----------------------------------------------------------
# R-code adapted from function plotcompmvr
#  (P. Filzmoser, package "chemometrics")

 pcr.dcv.optcomp = table(a_opt_all)/sum(table(a_opt_all))
    plot(names(pcr.dcv.optcomp), pcr.dcv.optcomp, type = "b", 
        main="Distribution of optimum a-values", 
        xlab = "Number of components", 
        ylab = "Relative frequency for optimal number", 
        cex.lab = 1.2)
    optcomp = as.numeric(names(which.max(pcr.dcv.optcomp)))
    abline(v = optcomp, lty = 2)

# Plot 5: SEP versus no. of PLS components (lines)
# ------------------------------------------------
# Separate line for each repetition, bold line = mean

#  SEP_all     SEP for all no. of components and all repetitions
#               matrix[components, repetitions]           

SEPmean = apply(SEP_all,1,mean) # means over repetitions
SEPmin = min(SEP_all)
SEPmax = max(SEP_all)

plot(0, 0,main="SEP versus a", type="l", lty=1, 
     col="black", xlab="Number of components", ylab="SEP", cex.lab=1.2,
     xlim = c(1,amax_pls),
     ylim = c(min(SEP_all),max(SEP_all)) )

for (r in 1:repetitions) {
    lines(1:amax_pls,SEP_all[,r], col=gray(0.6), lwd= 1) }
lines(1:amax_pls,SEPmean, col="black", lwd=2)

abline(v = a_final, lty = 2)       # vertical line at a_final
abline(h = SEP_final, lty = 2)     # horizontal line at SEP_final

# Plot 6: Distribution of SEP-values (boxplots) versus no. of PLS comp.
# --------------------------------------------------------------------

SEP_all_boxplot = as.data.frame(t(SEP_all))   # data frame necessary 
                                              # for multiple boxplot         
colnames(SEP_all_boxplot) = seq(1,ncol(t(SEP_all)))          # corr. 140214
boxplot(SEP_all_boxplot,
        main = "SEP distribution versus a", 
        xlab = "Number of components", 
        ylab = "SEP",
        lty = 1,cex.lab=1.2 )

# Plot 7: Distribution of SEP at a_final PLS components (1 boxplot)
# -----------------------------------------------------------------

boxplot(SEP_all[a_final,],                                    # corr. 140214
        main = paste("SEP distribution for a_final =",a_final), 
        xlab = paste(repetitions," SEP values \n each from n =",
               n, "test set residuals"),        
        ylab = "SEP",
        lty = 1,cex.lab=1.2 )

# Plot 8: Distribution of corr. coeff. at a_final PLS components (1 boxplot)
# --------------------------------------------------------------------------

boxplot(corr_all[,a_final],
        main = paste("Corr.coeff. distribution for a_final =",a_final), 
        xlab = paste(repetitions," corr. coeff. \n each from n =",
               n, "test set residuals"),        
        ylab = "Corr. coeff.",
        lty = 1,cex.lab=1.2 )

# Plot 9: Distribution of bias-values (boxplots) versus no. of PLS comp.
# ---------------------------------------------------------------------

# bias_all = matrix[1:repetitions,1:amax_pls]

bias_all_boxplot = as.data.frame(bias_all)  # data frame necessary
                                            # for multiple boxplot
         
colnames(bias_all_boxplot) = seq(1,amax_pls)
boxplot(bias_all_boxplot,
        main = "Bias distribution versus a", 
        xlab = "Number of components", 
        ylab = "Bias",
        lty = 1,cex.lab=1.2 )

# Plot 10: Distribution of bias at a_final PLS components (1 boxplot) 
# -------------------------------------------------------------------

boxplot(bias_all[,a_final],
        main = paste("Bias distribution for a_final =",a_final), 
        xlab = paste(repetitions," bias values \n each from n =",
               n, "test set residuals"),        
        ylab = "Bias",
        lty = 1,cex.lab=1.2 )

# ---
dev.off()
if (messages>0) {cat(" | closed")}

}  # end if PDFfile ~~~~~~~~~~~~~~~~~~~~~~

comptime = proc.time() - comptime
if (messages>0) {cat("\n=== End of ",version,
              "      ",comptime[1],"s \n")}

return(result)

} # end function


