varsel_almost_const <- function(X, k=3, round_to=4, messages=TRUE) {

# Select variables (columns in numerical matrix X) that are not constant or 
#  not almost constant.

# Almost constant means all values are constant except a maximum of k values
#   (which are not NA).  k <= n/2, n is the number of objects (rows).

# Matrix X ist first rounded to round_to decimals.

#  ARGUMENTS
#  =========

#  X            numerical matrix (n rows, m columns)

#  k            if all values of a variable but k or less are constant,
#                 the variable will be deleted; 
#               error stop if k >= n/2

#  round_to     before checking the variables are rounded to round_to decimals

#  messages     messages on screen if TRUE 

#  VALUES
#  ======

#  var_not_almost_const   result vector (logical, m components)
#                         TRUE if variable not constant or almost constant
#                              (selected),
#                         FALSE if variable is constant or almost constant
#                              (not selected) 


version = "varsel_almost_const 130222 VK"    # K. Varmuza, TU Vienna

n = nrow(X)
m = ncol(X)

if (messages) {
cat("\nStart: ",version)
cat("\nSearch for variables (columns in X) that are constant or almost constant.")
cat("\n Almost constant means all values are constant except a maximum of k values")
cat("\n (which are not NA).")
cat("\n X(",n,"x",m,")   k =",k,"  round_to =",round_to,"decimals")
}
if(n <= k*2) stop("Error: k <= n/2 not allowed")

X = round(X,round_to)

j <- rep(TRUE,m)    # algorithm by Leonhard Seyfang (TU Vienna), Aug 2006
for(i in 1:m) {    
     temp <- sort(X[,i], na.last=TRUE)
     if(is.na(temp[floor(n/2)+1])) {
         if(sum(is.na(temp)) >= n-k) j[i] <- FALSE	
     } else {
         if(sum(temp == temp[floor(sum(!is.na(temp))/2)+1], na.rm=TRUE) 
             + sum(is.na(temp)) >= n-k) {j[i] <- FALSE}
     }
 }
mj = sum(j)     # no. of remaining variables
if (messages) {
    cat("\n",(m-mj)," variables deleted, ",mj," variables remain. End. \n") }

var_not_almost_const = j        # logical result vector
return(var_not_almost_const)
    
}  # end of function
    
