varsel_corr_xx = function(X, method="pearson", r2limit=0.9, 
                             plot_histo=FALSE, messages=TRUE) {

# Variable selection: max. correlation between x-variables

# Method: All variable pairs are checked; if the correlation measure is above
#         r2limit, then one of the variables is marked FALSE (it is the variable
#         with the higher sum of the correlation measures to all variables).

#  ARGUMENTS
#  =========

#  X              numerical matrix (n rows, m columns)

#  method        "pearson" or "spearman" or "kendall"
  
#  r2limit        limit of squared correlation measure
#                 If correlation between 2 variables > r2limit one of them is
#                  deleted. The deleted variable has the higher sum of squared
#                  correlation measures to all other variables.

#  plot_histo     if TRUE: plot histogram of all correlation measures
#                   (for lower triangle of matrix with correlation measures) 

#  messages       messages on screen if TRUE
 
#  VALUES
#  ======

#  var_not_xcorr   result vector (logical, m components)
#                    TRUE   variable selected
#                    FALSE  variable deleted     

version = "varsel_corr_xx 130222 VK"        # K. Varmuza, TU Vienna

n = nrow(X)
m = ncol(X)
if (messages) {
  cat("\nStart: ",version)
  cat("\nDelete variables (columns in X) that have a squared corr. measure >",
         r2limit,"\n with any other variable. Method:",method)
  cat("\n X(",n,"x",m,")")
}

r = cor(X, method=method)    # corrr. coeff. matrix
r = r**2                     # squared, element-wise
rmin=min(r)
rmax=max(as.vector(r[lower.tri(r)]))
# cat("\n Squared corr. measures in X:",rmin,"...",rmax)

if (plot_histo) {
    plot(hist(as.vector(r[lower.tri(r)])),
       xlab=paste("Squared corr. measure:",round(rmin,4),"...",round(rmax,4)),
       main=paste(method," of X(",n,"x",m,")",sep=""))
 }

sel = rep(TRUE,m)        # selected variables init.
rsum = rowSums(r)
# cat("\nrowSums=",rsum)

for (j in 1:m) {
  for (jj in 1:m) {
    if(r[j,jj]>r2limit & j!=jj) {
       if (rsum[jj]>rsum[j]) {sel[jj]=FALSE}
                        else {sel[j] =FALSE} }
   }                   
 }

mj = sum(sel)     # no. of remaining variables
if (messages) {
    cat("\n",(m-mj)," variables deleted, ",mj," variables remain. End. \n") }

var_not_xcorr = sel        # logical result vector
return(var_not_xcorr)
 
}  # end function
